"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getConfig = getConfig;
exports.loadConfig = loadConfig;

var _defaultConf = require("./default.conf.js");

var _path = _interopRequireDefault(require("path"));

var _logger = require("@tractor/logger");

var _lodash = _interopRequireDefault(require("lodash.defaults"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Constants:
var CONFIG_FILE_NAME = 'tractor.conf.js';
var EMPTY_TAG = ''; // Utilites:

var config;

function getConfig(configPath) {
  config = config || loadConfig(configPath);
  return config;
}

function loadConfig() {
  var configPath = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : CONFIG_FILE_NAME;
  (0, _logger.info)('Loading config...');
  configPath = _path.default.resolve(process.cwd(), configPath);
  var config;

  try {
    config = require(configPath);
    config = config.default ? config.default : config;
  } catch (e) {
    config = {};
  }

  config = (0, _lodash.default)(config, _defaultConf.DEFAULT_CONFIG);
  config.tags = sortTags(config.tags);
  return config;
}

function sortTags(tags) {
  // Remove empty tag
  if (tags.includes(EMPTY_TAG)) {
    var index = tags.indexOf(EMPTY_TAG);
    tags.splice(index, 1);
  } // Add @s


  tags = tags.map(function (tag) {
    return tag.startsWith('@') ? tag : "@".concat(tag);
  }); // Add empty tag to start

  tags.unshift(EMPTY_TAG);
  return tags;
}