"use strict";

var _unitTest = require("@tractor/unit-test");

var _path = _interopRequireDefault(require("path"));

var tractorLogger = _interopRequireWildcard(require("@tractor/logger"));

var _index = require("./index");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global describe:true, it:true */
// Test setup:
// Dependencies:
// Under test:
describe('@tractor/config-loader:', function () {
  describe('getConfig:', function () {
    it('should re-use the config if it has already been loaded', function () {
      _unitTest.sinon.stub(tractorLogger, 'info');

      var config1 = (0, _index.getConfig)();
      var config2 = (0, _index.getConfig)();
      (0, _unitTest.expect)(config1).to.equal(config2);
      tractorLogger.info.restore();
    });
  });
  describe('loadConfig:', function () {
    it('should use the passed in path to load the config', function () {
      _unitTest.sinon.stub(process, 'cwd').returns('.');

      _unitTest.sinon.spy(_path.default, 'resolve');

      _unitTest.sinon.stub(tractorLogger, 'info');

      (0, _index.loadConfig)('./my.conf.js');
      (0, _unitTest.expect)(_path.default.resolve).to.have.been.calledWith('.', './my.conf.js');
      process.cwd.restore();

      _path.default.resolve.restore();

      tractorLogger.info.restore();
    });
    it('should fall back to looking from "./tractor.conf.js"', function () {
      _unitTest.sinon.stub(process, 'cwd').returns('.');

      _unitTest.sinon.spy(_path.default, 'resolve');

      _unitTest.sinon.stub(tractorLogger, 'info');

      (0, _index.loadConfig)();
      (0, _unitTest.expect)(_path.default.resolve).to.have.been.calledWith('.', 'tractor.conf.js');
      process.cwd.restore();

      _path.default.resolve.restore();

      tractorLogger.info.restore();
    });
    it('should load config values from a file', function () {
      _unitTest.sinon.stub(tractorLogger, 'info');

      var config = (0, _index.loadConfig)('./assets/test.conf.js');
      (0, _unitTest.expect)(config.port).to.equal(5000);
      (0, _unitTest.expect)(config.directory).to.equal('./tests/e2e');
      tractorLogger.info.restore();
    });
    it('should load config values from an ES2015 module', function () {
      _unitTest.sinon.stub(tractorLogger, 'info');

      var config = (0, _index.loadConfig)('./assets/test.esm.conf.js');
      (0, _unitTest.expect)(config.port).to.equal(5000);
      (0, _unitTest.expect)(config.directory).to.equal('./tests/e2e');
      tractorLogger.info.restore();
    });
    it('should load any missing values from the default config', function () {
      _unitTest.sinon.stub(tractorLogger, 'info');

      var config = (0, _index.loadConfig)();
      (0, _unitTest.expect)(config.port).to.equal(4000);
      (0, _unitTest.expect)(config.directory).to.equal('./tractor');
      (0, _unitTest.expect)(config.environments).to.deep.equal(['http://localhost:8080']);
      (0, _unitTest.expect)(config.tags).to.deep.equal(['']);
      tractorLogger.info.restore();
    });
    it('should make the empty tag the first tag', function () {
      _unitTest.sinon.stub(tractorLogger, 'info');

      var config = (0, _index.loadConfig)('./assets/test.tags.conf.js');
      (0, _unitTest.expect)(config.tags).to.deep.equal(['', '@foo', '@bar']);
      tractorLogger.info.restore();
    });
  });
});