"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Container = void 0;

var _util = require("util");

var _errorHandler = require("@tractor/error-handler");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Container =
/*#__PURE__*/
function () {
  function Container() {
    _classCallCheck(this, Container);

    this._constants = {};
    this._factories = {};
  }

  _createClass(Container, [{
    key: "call",
    value: function call(func, args) {
      var toInject = func['@Inject'] || [];
      var dependencies = get.call(this, toInject);
      return func.apply(null, dependencies.concat(args));
    }
  }, {
    key: "constant",
    value: function constant(constants) {
      var _this = this;

      Object.keys(constants).forEach(function (key) {
        if (_this._constants[key]) {
          throw new _errorHandler.TractorError("Cannot register \"".concat(key, "\" with DI container, because there is already a constant registered with that name"));
        }

        _this._constants[key] = constants[key];
      });
    }
  }, {
    key: "factory",
    value: function factory(_factory) {
      if (!(0, _util.isFunction)(_factory)) {
        throw new _errorHandler.TractorError('Cannot register factory with DI container, because it is not a function');
      }

      if (!_factory.name) {
        throw new _errorHandler.TractorError('Cannot register factory with DI container, because it is an anonymous function');
      }

      if (this._factories[_factory.name]) {
        throw new _errorHandler.TractorError("Cannot register \"".concat(_factory.name, "\" with DI container, because there is already a factory registered with that name"));
      }

      this._factories[_factory.name] = _factory;
    }
  }, {
    key: "instantiate",
    value: function instantiate(factory) {
      var toInject = factory['@Inject'] || [];
      var dependencies = get.call(this, toInject);
      return new (Function.prototype.bind.apply(factory, [null].concat(dependencies)))();
    }
  }]);

  return Container;
}();

exports.Container = Container;

function get(dependencies) {
  var _this2 = this;

  return dependencies.map(function (dependency) {
    var constant = _this2._constants[dependency];
    return constant || _this2.instantiate(lookup.call(_this2, dependency));
  });
}

function lookup(dependency) {
  var factory = this._factories[dependency];

  if (!factory) {
    throw new _errorHandler.TractorError("Could not find a factory for \"".concat(dependency, "\""));
  }

  return factory;
}