"use strict";

var _constants = require("./constants");

var _unitTest = require("@tractor/unit-test");

var _TractorError = require("./TractorError");

/* global describe:true, it:true */
// Constants:
// Test setup:
// Under test:
describe('@tractor/error-handler: TractorError:', function () {
  describe('TractorError constructor:', function () {
    it('should create a new TractorError', function () {
      var error = new _TractorError.TractorError();
      (0, _unitTest.expect)(error).to.be.an.instanceof(_TractorError.TractorError);
      (0, _unitTest.expect)(error.name).to.equal('TractorError');
    });
    it('should inherit from Error', function () {
      var error = new _TractorError.TractorError();
      (0, _unitTest.expect)(error).to.be.an.instanceof(Error);
    });
    it('should have an error message', function () {
      var error = new _TractorError.TractorError('message');
      (0, _unitTest.expect)(error.message).to.equal('message');
    });
    it('should have an default status of 500', function () {
      var error = new _TractorError.TractorError();
      (0, _unitTest.expect)(error.status).to.equal(_constants.SERVER_ERROR);
    });
    it('should capture the stack trace', function () {
      _unitTest.sinon.stub(Error, 'captureStackTrace');
      /* eslint-disable */


      var error = new _TractorError.TractorError();
      /* eslint-enable */

      (0, _unitTest.expect)(Error.captureStackTrace).to.have.been.called();
      Error.captureStackTrace.restore();
    });
  });
  describe('TractorError.isTractorError:', function () {
    it('should be `true` for instances of TractorError', function () {
      (0, _unitTest.expect)(_TractorError.TractorError.isTractorError(new _TractorError.TractorError())).to.equal(true);
    });
    it('should be `true` for TractorError-like objects', function () {
      (0, _unitTest.expect)(_TractorError.TractorError.isTractorError({
        _isTractorError: true
      })).to.equal(true);
    });
    it('should be `false` for non-TractorError objects', function () {
      (0, _unitTest.expect)(_TractorError.TractorError.isTractorError({})).to.equal(false);
    });
  });
});