"use strict";

var _constants = require("./constants");

var _unitTest = require("@tractor/unit-test");

var tractorLogger = _interopRequireWildcard(require("@tractor/logger"));

var _handleError = require("./handle-error");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

describe('@tractor/error-handler: handle-error:', function () {
  it('should log the error to the console', function () {
    _unitTest.sinon.stub(tractorLogger, 'error');

    var response = {
      status: function status() {},
      send: function send() {}
    };
    (0, _handleError.handleError)(response, new Error(), 'error');
    (0, _unitTest.expect)(tractorLogger.error).to.have.been.calledWith('error');
    tractorLogger.error.restore();
  });
  it('should fall back to the `message` from the given `error`', function () {
    _unitTest.sinon.stub(tractorLogger, 'error');

    var response = {
      status: function status() {},
      send: function send() {}
    };
    (0, _handleError.handleError)(response, new Error('error'));
    (0, _unitTest.expect)(tractorLogger.error).to.have.been.calledWith('error');
    tractorLogger.error.restore();
  });
  it('should have a `status` of `500`', function () {
    _unitTest.sinon.stub(tractorLogger, 'error');

    var response = {
      status: function status() {},
      send: function send() {}
    };

    _unitTest.sinon.spy(response, 'status');

    (0, _handleError.handleError)(response, new Error());
    (0, _unitTest.expect)(response.status).to.have.been.calledWith(_constants.SERVER_ERROR);
    tractorLogger.error.restore();
  });
  it('should response with the `message`', function () {
    _unitTest.sinon.stub(tractorLogger, 'error');

    var response = {
      status: function status() {},
      send: function send() {}
    };

    _unitTest.sinon.spy(response, 'send');

    (0, _handleError.handleError)(response, new Error(), 'error');

    var _response$send$firstC = _slicedToArray(response.send.firstCall.args, 1),
        responseData = _response$send$firstC[0];

    (0, _unitTest.expect)(JSON.parse(responseData).error).to.equal('error');
    tractorLogger.error.restore();
  });
  it('should fall back to the `message` from the given `error`', function () {
    _unitTest.sinon.stub(tractorLogger, 'error');

    var response = {
      status: function status() {},
      send: function send() {}
    };

    _unitTest.sinon.spy(response, 'send');

    (0, _handleError.handleError)(response, new Error('error'));

    var _response$send$firstC2 = _slicedToArray(response.send.firstCall.args, 1),
        responseData = _response$send$firstC2[0];

    (0, _unitTest.expect)(JSON.parse(responseData).error).to.equal('error');
    tractorLogger.error.restore();
  });
});