"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDeleteItemHandler = createDeleteItemHandler;

var _Directory = require("../structure/Directory");

var _utilities = require("./utilities");

var _errorHandler = require("@tractor/error-handler");

// Dependencies:
// Errors:
function createDeleteItemHandler(fileStructure) {
  return function deleteItem(request, response) {
    var _request$query = request.query,
        cleanup = _request$query.cleanup,
        rimraf = _request$query.rimraf;
    var itemUrl = request.params[0];
    var itemPath = (0, _utilities.getItemPath)(fileStructure, itemUrl);
    var toDelete = fileStructure.allFilesByPath[itemPath] || fileStructure.allDirectoriesByPath[itemPath];

    if (!toDelete) {
      return (0, _utilities.respondItemNotFound)(itemPath, response);
    }

    var operation;

    if (rimraf && toDelete instanceof _Directory.Directory) {
      operation = toDelete.rimraf();
    } else if (cleanup) {
      operation = toDelete.cleanup();
    } else {
      operation = toDelete.delete();
    }

    return operation.then(function () {
      return (0, _utilities.respondOkay)(response);
    }).catch(_errorHandler.TractorError.isTractorError, function (error) {
      return (0, _errorHandler.handleError)(response, error);
    }).catch(function () {
      return (0, _errorHandler.handleError)(response, new _errorHandler.TractorError("Could not delete \"".concat(itemPath, "\"")));
    });
  };
}