"use strict";

var _testSetup = require("../../test-setup");

var _path = _interopRequireDefault(require("path"));

var _Directory = require("../structure/Directory");

var _File = require("../structure/File");

var _FileStructure = require("../structure/FileStructure");

var utilities = _interopRequireWildcard(require("./utilities"));

var tractorErrorHandler = _interopRequireWildcard(require("@tractor/error-handler"));

var _deleteItem = require("./delete-item");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global describe:true, it:true, xit:true */
// Test setup:
// Dependencies:
// Errors:
// Under test:
describe('@tractor/file-structure - actions/delete-item:', function () {
  it('should delete a file', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      params: [file.url],
      query: {}
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'delete').resolves();

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(_File.File.prototype.delete).to.have.been.called();
    }).finally(function () {
      _File.File.prototype.delete.restore();
    });
  });
  it('should delete a directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
    var request = {
      params: [directory.url],
      query: {}
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'delete').resolves();

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(_Directory.Directory.prototype.delete).to.have.been.called();
    }).finally(function () {
      _Directory.Directory.prototype.delete.restore();
    });
  });
  it("should throw an error if it can't find the item to delete", function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var request = {
      params: ['/directory/missing-item'],
      query: {}
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(utilities, 'respondItemNotFound');

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    deleteItem(request, response);
    (0, _testSetup.expect)(utilities.respondItemNotFound).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'missing-item'), response);
    utilities.respondItemNotFound.restore();
  });
  it('should rimraf a directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
    var request = {
      params: [directory.url],
      query: {
        rimraf: true
      }
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'rimraf').resolves();

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(_Directory.Directory.prototype.rimraf).to.have.been.called();
    }).finally(function () {
      _Directory.Directory.prototype.rimraf.restore();
    });
  });
  it("should respond with OK", function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      params: [file.url],
      query: {}
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'delete').resolves();

    _testSetup.sinon.stub(response, 'sendStatus');

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(response.sendStatus).to.have.been.calledWith(200);
    }).finally(function () {
      _File.File.prototype.delete.restore();
    });
  });
  it('should fall back to delete if `rimraf` is passed but the item is not a directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      params: [file.url],
      query: {
        rimraf: true
      }
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'delete').resolves();

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(_File.File.prototype.delete).to.have.been.called();
    }).finally(function () {
      _File.File.prototype.delete.restore();
    });
  });
  it('should cleanup a file', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      params: [file.url],
      query: {
        cleanup: true
      }
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'cleanup').resolves();

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(_File.File.prototype.cleanup).to.have.been.called();
    }).finally(function () {
      _File.File.prototype.cleanup.restore();
    });
  });
  it('should cleanup a directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
    var request = {
      params: [directory.url],
      query: {
        cleanup: true
      }
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'cleanup').resolves();

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(_Directory.Directory.prototype.cleanup).to.have.been.called();
    }).finally(function () {
      _Directory.Directory.prototype.cleanup.restore();
    });
  });
  xit('should handle known TractorErrors', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      params: [file.url],
      query: {}
    };
    var response = {
      sendStatus: function sendStatus() {}
    };
    var error = new tractorErrorHandler.TractorError();

    _testSetup.sinon.stub(_File.File.prototype, 'delete').rejects(error);

    _testSetup.sinon.stub(tractorErrorHandler, 'handleError');

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(_File.File.prototype.delete).to.have.been.called();
      (0, _testSetup.expect)(tractorErrorHandler.handleError).to.have.been.calledWith(response, error);
    }).finally(function () {
      _File.File.prototype.delete.restore();

      tractorErrorHandler.handleError.restore();
    });
  });
  xit('should handle unknown errors', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      params: [file.url],
      query: {}
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'delete').rejects();

    _testSetup.sinon.stub(tractorErrorHandler, 'handleError');

    var deleteItem = (0, _deleteItem.createDeleteItemHandler)(fileStructure);
    return deleteItem(request, response).then(function () {
      (0, _testSetup.expect)(tractorErrorHandler.handleError).to.have.been.calledWith(response, new tractorErrorHandler.TractorError("Could not delete \"".concat(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), "\"")));
    }).finally(function () {
      _File.File.prototype.delete.restore();

      tractorErrorHandler.handleError.restore();
    });
  });
});