"use strict";

var _testSetup = require("../../test-setup");

var _path = _interopRequireDefault(require("path"));

var _Directory = require("../structure/Directory");

var _File = require("../structure/File");

var _FileStructure = require("../structure/FileStructure");

var utilities = _interopRequireWildcard(require("./utilities"));

var tractorErrorHandler = _interopRequireWildcard(require("@tractor/error-handler"));

var _moveItem = require("./move-item");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global describe:true, it:true, xit:true */
// Test setup:
// Dependencies:
// Dependencies:
// Under test:
describe('@tractor/file-structure - actions/move-item:', function () {
  it('should move a file', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {
        newUrl: _path.default.join(_path.default.sep, 'other-directory', 'file.ext')
      },
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'move').resolves();

    var moveItem = (0, _moveItem.createMoveItemHandler)(fileStructure);
    return moveItem(request, response).then(function () {
      (0, _testSetup.expect)(_File.File.prototype.move).to.have.been.calledWith({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'file.ext')
      });
    }).finally(function () {
      _File.File.prototype.move.restore();
    });
  });
  it('should copy a file', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {
        copy: true
      },
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'move').resolves();

    var moveItem = (0, _moveItem.createMoveItemHandler)(fileStructure);
    return moveItem(request, response).then(function () {
      (0, _testSetup.expect)(_File.File.prototype.move).to.have.been.calledWith({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'file (1).ext')
      });
    }).finally(function () {
      _File.File.prototype.move.restore();
    });
  });
  it('should move a directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
    var request = {
      body: {
        newUrl: _path.default.join(_path.default.sep, 'other-directory')
      },
      params: [directory.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'move').resolves();

    var moveItem = (0, _moveItem.createMoveItemHandler)(fileStructure);
    return moveItem(request, response).then(function () {
      (0, _testSetup.expect)(_Directory.Directory.prototype.move).to.have.been.calledWith({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory')
      });
    }).finally(function () {
      _Directory.Directory.prototype.move.restore();
    });
  });
  it('should copy a directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
    var request = {
      body: {
        copy: true
      },
      params: [directory.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'move').resolves();

    var moveItem = (0, _moveItem.createMoveItemHandler)(fileStructure);
    return moveItem(request, response).then(function () {
      (0, _testSetup.expect)(_Directory.Directory.prototype.move).to.have.been.calledWith({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'directory (1)')
      });
    }).finally(function () {
      _Directory.Directory.prototype.move.restore();
    });
  });
  it("should respond with OK", function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {
        newUrl: _path.default.join(_path.default.sep, 'other-directory', 'file.ext')
      },
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'move').resolves();

    _testSetup.sinon.stub(response, 'sendStatus');

    var moveItem = (0, _moveItem.createMoveItemHandler)(fileStructure);
    return moveItem(request, response).then(function () {
      (0, _testSetup.expect)(response.sendStatus).to.have.been.calledWith(200);
    }).finally(function () {
      _File.File.prototype.move.restore();
    });
  });
  it("should throw an error if it can't find the item to move", function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var request = {
      body: {
        newUrl: _path.default.join(_path.default.sep, 'other-directory', 'missing-item')
      },
      params: ['/directory/missing-item']
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(utilities, 'respondItemNotFound');

    var moveItem = (0, _moveItem.createMoveItemHandler)(fileStructure);
    moveItem(request, response);
    (0, _testSetup.expect)(utilities.respondItemNotFound).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'missing-item'), response);
    utilities.respondItemNotFound.restore();
  });
  xit('should handle known TractorErrors', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {
        newUrl: _path.default.join(_path.default.sep, 'other-directory', 'file.ext')
      },
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };
    var error = new tractorErrorHandler.TractorError();

    _testSetup.sinon.stub(_File.File.prototype, 'move').rejects(error);

    _testSetup.sinon.stub(tractorErrorHandler, 'handleError');

    var moveItem = (0, _moveItem.createMoveItemHandler)(fileStructure);
    return moveItem(request, response).then(function () {
      (0, _testSetup.expect)(tractorErrorHandler.handleError).to.have.been.calledWith(response, error);
    }).finally(function () {
      _File.File.prototype.move.restore();

      tractorErrorHandler.handleError.restore();
    });
  });
  xit('should handle unknown errors', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {
        newUrl: _path.default.join(_path.default.sep, 'other-directory', 'file.ext')
      },
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'move').rejects();

    _testSetup.sinon.stub(tractorErrorHandler, 'handleError');

    var moveItem = (0, _moveItem.createMoveItemHandler)(fileStructure);
    return moveItem(request, response).then(function () {
      (0, _testSetup.expect)(tractorErrorHandler.handleError).to.have.been.calledWith(response, new tractorErrorHandler.TractorError("Could not move \"".concat(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), "\"")));
    }).finally(function () {
      _File.File.prototype.move.restore();

      tractorErrorHandler.handleError.restore();
    });
  });
});