"use strict";

var _testSetup = require("../../test-setup");

var _path = _interopRequireDefault(require("path"));

var _Directory = require("../structure/Directory");

var _File = require("../structure/File");

var _FileStructure = require("../structure/FileStructure");

var utilities = _interopRequireWildcard(require("./utilities"));

var _openItem = require("./open-item");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global describe:true, it:true */
// Test setup:
// Dependencies:
// Under test:
describe('@tractor/file-structure - actions/open-item:', function () {
  it('should open a file', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file.ext'), fileStructure);
    var serialised = {};
    var request = {
      params: [file.url]
    };
    var response = {
      send: function send() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'serialise').returns(serialised);

    _testSetup.sinon.stub(response, 'send');

    var openItem = (0, _openItem.createOpenItemHandler)(fileStructure);
    openItem(request, response);
    (0, _testSetup.expect)(_File.File.prototype.serialise).to.have.been.called();
    (0, _testSetup.expect)(response.send).to.have.been.calledWith(serialised);

    _File.File.prototype.serialise.restore();
  });
  it('should open a directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
    var serialised = {};
    var request = {
      params: [directory.url]
    };
    var response = {
      send: function send() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'serialise').returns(serialised);

    _testSetup.sinon.stub(response, 'send');

    var openItem = (0, _openItem.createOpenItemHandler)(fileStructure);
    openItem(request, response);
    (0, _testSetup.expect)(_Directory.Directory.prototype.serialise).to.have.been.called();
    (0, _testSetup.expect)(response.send).to.have.been.calledWith(serialised);

    _Directory.Directory.prototype.serialise.restore();
  });
  it("should throw an error if it can't find the file to open", function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var request = {
      params: ['/directory/missing-item']
    };
    var response = {
      send: function send() {}
    };

    _testSetup.sinon.stub(utilities, 'respondItemNotFound');

    var openItem = (0, _openItem.createOpenItemHandler)(fileStructure);
    openItem(request, response);
    (0, _testSetup.expect)(utilities.respondItemNotFound).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'missing-item'), response);
    utilities.respondItemNotFound.restore();
  });
});