"use strict";

var _testSetup = require("../../test-setup");

var _path = _interopRequireDefault(require("path"));

var _File = require("../structure/File");

var _FileStructure = require("../structure/FileStructure");

var utilities = _interopRequireWildcard(require("./utilities"));

var tractorErrorHandler = _interopRequireWildcard(require("@tractor/error-handler"));

var _refactorItem = require("./refactor-item");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global describe:true, it:true, xit:true */
// Test setup:
// Dependencies:
// Errors:
// Under test:
describe('@tractor/file-structure - actions/refactor-item:', function () {
  it('should refactor a file', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {},
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'refactor').resolves();

    var refactorItem = (0, _refactorItem.createRefactorItemHandler)(fileStructure);
    return refactorItem(request, response).then(function () {
      (0, _testSetup.expect)(_File.File.prototype.refactor).to.have.been.called();
    }).finally(function () {
      _File.File.prototype.refactor.restore();
    });
  });
  it("should throw an error if it can't find the file to refactor", function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var request = {
      body: {},
      params: ['/directory/missing-item']
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(utilities, 'respondItemNotFound');

    var refactorItem = (0, _refactorItem.createRefactorItemHandler)(fileStructure);
    refactorItem(request, response);
    (0, _testSetup.expect)(utilities.respondItemNotFound).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'missing-item'), response);
    utilities.respondItemNotFound.restore();
  });
  it("should respond with OK", function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {},
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'refactor').resolves();

    _testSetup.sinon.stub(response, 'sendStatus');

    var refactorItem = (0, _refactorItem.createRefactorItemHandler)(fileStructure);
    return refactorItem(request, response).then(function () {
      (0, _testSetup.expect)(response.sendStatus).to.have.been.calledWith(200);
    }).finally(function () {
      _File.File.prototype.refactor.restore();
    });
  });
  xit('should handle known TractorErrors', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {},
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };
    var error = new tractorErrorHandler.TractorError();

    _testSetup.sinon.stub(_File.File.prototype, 'refactor').rejects(error);

    _testSetup.sinon.stub(tractorErrorHandler, 'handleError');

    var refactorItem = (0, _refactorItem.createRefactorItemHandler)(fileStructure);
    return refactorItem(request, response).then(function () {
      (0, _testSetup.expect)(_File.File.prototype.refactor).to.have.been.called();
      (0, _testSetup.expect)(tractorErrorHandler.handleError).to.have.been.calledWith(response, error);
    }).finally(function () {
      _File.File.prototype.refactor.restore();

      tractorErrorHandler.handleError.restore();
    });
  });
  xit('should handle unknown errors', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {},
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File.File.prototype, 'refactor').rejects();

    _testSetup.sinon.stub(tractorErrorHandler, 'handleError');

    var refactorItem = (0, _refactorItem.createRefactorItemHandler)(fileStructure);
    return refactorItem(request, response).then(function () {
      (0, _testSetup.expect)(tractorErrorHandler.handleError).to.have.been.calledWith(response, new tractorErrorHandler.TractorError("Could not refactor \"".concat(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), "\"")));
    }).finally(function () {
      _File.File.prototype.refactor.restore();

      tractorErrorHandler.handleError.restore();
    });
  });
});