"use strict";

var _testSetup = require("../../test-setup");

var _path = _interopRequireDefault(require("path"));

var _Directory = require("../structure/Directory");

var _File5 = require("../structure/File");

var _FileStructure = require("../structure/FileStructure");

var tractorErrorHandler = _interopRequireWildcard(require("@tractor/error-handler"));

var _saveItem = require("./save-item");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

describe('@tractor/file-structure - actions/save-item:', function () {
  it('should save a file', function () {
    var TestFile =
    /*#__PURE__*/
    function (_File) {
      _inherits(TestFile, _File);

      function TestFile() {
        _classCallCheck(this, TestFile);

        return _possibleConstructorReturn(this, (TestFile.__proto__ || Object.getPrototypeOf(TestFile)).apply(this, arguments));
      }

      _createClass(TestFile, [{
        key: "save",
        value: function save() {}
      }]);

      return TestFile;
    }(_File5.File);

    TestFile.prototype.extension = '.ext';
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    fileStructure.addFileType(TestFile);
    var request = {
      body: {
        data: 'data'
      },
      params: ['/directory/file.ext']
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'save').resolves();

    _testSetup.sinon.stub(TestFile.prototype, 'save').resolves();

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(TestFile.prototype.save).to.have.been.calledWith('data');
    }).finally(function () {
      _Directory.Directory.prototype.save.restore();
    });
  });
  it('should save a new file with multiple extensions', function () {
    var SpecialTestFile =
    /*#__PURE__*/
    function (_File2) {
      _inherits(SpecialTestFile, _File2);

      function SpecialTestFile() {
        _classCallCheck(this, SpecialTestFile);

        return _possibleConstructorReturn(this, (SpecialTestFile.__proto__ || Object.getPrototypeOf(SpecialTestFile)).apply(this, arguments));
      }

      _createClass(SpecialTestFile, [{
        key: "save",
        value: function save() {}
      }]);

      return SpecialTestFile;
    }(_File5.File);

    SpecialTestFile.prototype.extension = '.special.ext';
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    fileStructure.addFileType(SpecialTestFile);
    var request = {
      body: {
        data: 'data'
      },
      params: ['/directory/file.special.ext']
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(SpecialTestFile.prototype, 'save').resolves();

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(SpecialTestFile.prototype.save).to.have.been.calledWith('data');
    });
  });
  it('should fall back to the default File', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var request = {
      body: {
        data: 'data'
      },
      params: ['/directory/file.ext']
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File5.File.prototype, 'save').resolves();

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(_File5.File.prototype.save).to.have.been.calledWith('data');
    }).finally(function () {
      _File5.File.prototype.save.restore();
    });
  });
  it('should save a copy of a file if it already exists', function () {
    var TestFile =
    /*#__PURE__*/
    function (_File3) {
      _inherits(TestFile, _File3);

      function TestFile() {
        _classCallCheck(this, TestFile);

        return _possibleConstructorReturn(this, (TestFile.__proto__ || Object.getPrototypeOf(TestFile)).apply(this, arguments));
      }

      _createClass(TestFile, [{
        key: "save",
        value: function save() {}
      }]);

      return TestFile;
    }(_File5.File);

    TestFile.prototype.extension = '.ext';
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    fileStructure.addFileType(TestFile);
    var file = new TestFile(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {
        data: 'data'
      },
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.spy(file, 'save');

    _testSetup.sinon.stub(TestFile.prototype, 'save').resolves();

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(file.save).to.not.have.been.called();
      (0, _testSetup.expect)(TestFile.prototype.save).to.have.been.calledWith('data');
    });
  });
  it('should overwrite an existing file', function () {
    var TestFile =
    /*#__PURE__*/
    function (_File4) {
      _inherits(TestFile, _File4);

      function TestFile() {
        _classCallCheck(this, TestFile);

        return _possibleConstructorReturn(this, (TestFile.__proto__ || Object.getPrototypeOf(TestFile)).apply(this, arguments));
      }

      _createClass(TestFile, [{
        key: "save",
        value: function save() {}
      }]);

      return TestFile;
    }(_File5.File);

    TestFile.prototype.extension = '.ext';
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    fileStructure.addFileType(TestFile);
    var file = new TestFile(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
    var request = {
      body: {
        data: 'data',
        overwrite: true
      },
      params: [file.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(TestFile.prototype, 'save').resolves();

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(TestFile.prototype.save).to.have.been.calledWith('data');
    });
  });
  it('should save a directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var request = {
      body: {},
      params: ['/directory']
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'save').resolves();

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(_Directory.Directory.prototype.save).to.have.been.called();
    }).finally(function () {
      _Directory.Directory.prototype.save.restore();
    });
  });
  it('should save a copy of a directory if it already exists', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
    var request = {
      body: {},
      params: [directory.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.spy(directory, 'save');

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'save').resolves();

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(directory.save).to.not.have.been.called();
      (0, _testSetup.expect)(_Directory.Directory.prototype.save).to.have.been.called();
    }).finally(function () {
      _Directory.Directory.prototype.save.restore();
    });
  });
  it('should overwrite an existing directory', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
    var request = {
      body: {
        overwrite: true
      },
      params: [directory.url]
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_Directory.Directory.prototype, 'save').resolves();

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(_Directory.Directory.prototype.save).to.have.been.called();
    }).finally(function () {
      _Directory.Directory.prototype.save.restore();
    });
  });
  xit('should handle known TractorErrors', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var error = new tractorErrorHandler.TractorError();
    var request = {
      body: {
        data: 'data'
      },
      params: ['/directory/file.ext']
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File5.File.prototype, 'save').rejects();

    _testSetup.sinon.stub(tractorErrorHandler, 'handleError');

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(tractorErrorHandler.handleError).to.have.been.calledWith(response, error);
    }).finally(function () {
      _File5.File.prototype.save.restore();

      tractorErrorHandler.handleError.restore();
    });
  });
  xit('should handle unknown errors', function () {
    var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
    var request = {
      body: {
        data: 'data'
      },
      params: ['/directory/file.ext']
    };
    var response = {
      sendStatus: function sendStatus() {}
    };

    _testSetup.sinon.stub(_File5.File.prototype, 'save').rejects();

    _testSetup.sinon.stub(tractorErrorHandler, 'handleError');

    var saveItem = (0, _saveItem.createSaveItemHandler)(fileStructure);
    return saveItem(request, response).then(function () {
      (0, _testSetup.expect)(tractorErrorHandler.handleError).to.have.been.calledWith(response, new tractorErrorHandler.TractorError("Could not save \"".concat(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), "\"")));
    }).finally(function () {
      _File5.File.prototype.save.restore();

      tractorErrorHandler.handleError.restore();
    });
  });
});