"use strict";

var _testSetup = require("../../test-setup");

var _gracefulFs = _interopRequireDefault(require("graceful-fs"));

var _path = _interopRequireDefault(require("path"));

var _File3 = require("./File");

var _FileStructure = require("./FileStructure");

var _errorHandler = require("@tractor/error-handler");

var _Directory = require("./Directory");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

describe('@tractor/file-structure - Directory:', function () {
  describe('Directory constructor:', function () {
    it('should create a new Directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      (0, _testSetup.expect)(directory).to.be.an.instanceof(_Directory.Directory);
    });
    it('should work out the directory name', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      (0, _testSetup.expect)(directory.name).to.equal('directory');
    });
    it('should work out the directory basename', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      (0, _testSetup.expect)(directory.basename).to.equal('directory');
    });
    it('should work out the URL to the directory from the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      (0, _testSetup.expect)(directory.url).to.equal('/directory');
      (0, _testSetup.expect)(fileStructure.structure.url).to.equal('/');
    });
    it('should work out the parent directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);
      (0, _testSetup.expect)(subdirectory.directory).to.equal(directory);
    });
    it("should create the parent directory if it doesn't exist", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory', 'sub-directory'), fileStructure);
      (0, _testSetup.expect)(subdirectory.directory).to.not.be.undefined();
      (0, _testSetup.expect)(fileStructure.allDirectoriesByPath[_path.default.join(_path.default.sep, 'file-structure', 'parent-directory')]).to.not.be.undefined();
      (0, _testSetup.expect)(fileStructure.allDirectoriesByPath[_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory')]).to.not.be.undefined();
    });
    it('should throw an error if the Directory path is outside the root of the FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      (0, _testSetup.expect)(function () {
        new _Directory.Directory(_path.default.join(_path.default.sep, 'outside'), fileStructure);
      }).to.throw(_errorHandler.TractorError, "Cannot create \"".concat(_path.default.join(_path.default.sep, 'outside'), "\" because it is outside of the root of the FileStructure"));
    });
  });
  describe('Directory.addItem:', function () {
    it('should add a file to the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      directory.addItem(file);
      (0, _testSetup.expect)(directory.files.length).to.equal(1);

      var _directory$files = _slicedToArray(directory.files, 1),
          expectedFile = _directory$files[0];

      (0, _testSetup.expect)(expectedFile).to.equal(file);
    });
    it('should not add the file if it has already been added', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      directory.addItem(file);
      directory.addItem(file);
      (0, _testSetup.expect)(directory.files.length).to.equal(1);
    });
    it('should add the file to the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(fileStructure, 'addItem');

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      directory.addItem(file);
      (0, _testSetup.expect)(fileStructure.addItem).to.have.been.calledWith(file);
    });
    it('should add a directory to the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);
      directory.addItem(subdirectory);
      (0, _testSetup.expect)(directory.directories.length).to.equal(1);

      var _directory$directorie = _slicedToArray(directory.directories, 1),
          expectedDirectory = _directory$directorie[0];

      (0, _testSetup.expect)(expectedDirectory).to.equal(subdirectory);
    });
    it('should not add the directory if it has already been added', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);
      directory.addItem(subdirectory);
      directory.addItem(subdirectory);
      (0, _testSetup.expect)(directory.directories.length).to.equal(1);
    });
    it('should add the directory to the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(fileStructure, 'addItem');

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);
      directory.addItem(subdirectory);
      (0, _testSetup.expect)(fileStructure.addItem).to.have.been.calledWith(subdirectory);
    });
  });
  describe('Directory.cleanup:', function () {
    it('should delete the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(directory, 'delete').resolves();

      _testSetup.sinon.stub(directory.directory, 'cleanup').resolves();

      return directory.cleanup().then(function () {
        (0, _testSetup.expect)(directory.delete).to.have.been.called();
      });
    });
    it('should cleanup the parent directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(directory, 'delete').resolves();

      _testSetup.sinon.stub(directory.directory, 'cleanup').resolves();

      return directory.cleanup().then(function () {
        (0, _testSetup.expect)(directory.directory.cleanup).to.have.been.called();
      });
    });
    it('should stop once it gets to the root of the FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.spy(directory, 'delete');

      _testSetup.sinon.spy(directory.directory, 'delete');

      return directory.cleanup().then(function () {
        (0, _testSetup.expect)(directory.delete).to.have.been.called();
        (0, _testSetup.expect)(directory.directory.delete).to.have.been.called();
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should stop once it gets to a directory that is not empty', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.spy(directory, 'delete');

      _testSetup.sinon.spy(file, 'delete');

      _testSetup.sinon.spy(subdirectory, 'delete');

      return subdirectory.cleanup().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.rmdirAsync).to.have.been.calledOnce();
        (0, _testSetup.expect)(subdirectory.delete).to.have.been.called();
        (0, _testSetup.expect)(directory.delete).to.have.been.called();
        (0, _testSetup.expect)(file.delete).to.not.have.been.called();
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should rethrow if something unexpected goes wrong', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var error = new Error('Unexpected error');

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').rejects(error);

      return directory.cleanup().catch(function (e) {
        (0, _testSetup.expect)(e).to.equal(error);
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
  });
  describe('Directory.delete:', function () {
    it('should delete the directory if it is empty', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      return directory.delete().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.rmdirAsync).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'directory'));
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should remove itself from its parent directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.stub(fileStructure.structure, 'removeItem');

      return directory.delete().then(function () {
        (0, _testSetup.expect)(fileStructure.structure.removeItem).to.have.been.calledWith(directory);
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should remove itself from the FileStructure if it is the root directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.stub(fileStructure, 'removeItem');

      return fileStructure.structure.delete().then(function () {
        (0, _testSetup.expect)(fileStructure.removeItem).to.have.been.calledWith(fileStructure.structure);
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should throw if the directory is not empty', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.spy(file, 'delete');

      _testSetup.sinon.spy(subdirectory, 'delete');

      return directory.delete().catch(function (e) {
        (0, _testSetup.expect)(e).to.deep.equal(new _errorHandler.TractorError("Cannot delete \"".concat(directory.path, "\" because it is not empty")));
        (0, _testSetup.expect)(subdirectory.delete).to.not.have.been.called();
        (0, _testSetup.expect)(file.delete).to.not.have.been.called();
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
  });
  describe('Directory.move:', function () {
    it('should move a directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.spy(_Directory.Directory.prototype, 'constructor');

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'save').resolves();

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      return directory.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory')
      }).then(function () {
        (0, _testSetup.expect)(_Directory.Directory.prototype.constructor).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'other-directory'), fileStructure);
        (0, _testSetup.expect)(_Directory.Directory.prototype.save).to.have.been.called();
        (0, _testSetup.expect)(_Directory.Directory.prototype.delete).to.have.been.called();
      }).finally(function () {
        _Directory.Directory.prototype.constructor.restore();

        _Directory.Directory.prototype.delete.restore();

        _Directory.Directory.prototype.save.restore();
      });
    });
    it('should copy a directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.spy(_Directory.Directory.prototype, 'constructor');

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'save').resolves();

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      return directory.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory')
      }, {
        isCopy: true
      }).then(function () {
        (0, _testSetup.expect)(_Directory.Directory.prototype.constructor).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'other-directory'), fileStructure);
        (0, _testSetup.expect)(_Directory.Directory.prototype.save).to.have.been.called();
        (0, _testSetup.expect)(_Directory.Directory.prototype.delete).to.not.have.been.called();
      }).finally(function () {
        _Directory.Directory.prototype.constructor.restore();

        _Directory.Directory.prototype.delete.restore();

        _Directory.Directory.prototype.save.restore();
      });
    });
    it('should move all the children items', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'save').resolves();

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subDirectory1 = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory-1'), fileStructure);
      var subDirectory2 = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory-2'), fileStructure);

      _testSetup.sinon.stub(subDirectory1, 'move').resolves();

      _testSetup.sinon.stub(subDirectory2, 'move').resolves();

      return directory.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory')
      }).then(function () {
        (0, _testSetup.expect)(subDirectory1.move).to.have.been.calledWith({
          newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'sub-directory-1')
        });
        (0, _testSetup.expect)(subDirectory2.move).to.have.been.calledWith({
          newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'sub-directory-2')
        });
      }).finally(function () {
        _Directory.Directory.prototype.delete.restore();

        _Directory.Directory.prototype.save.restore();
      });
    });
  });
  describe('Directory.read:', function () {
    it('should read the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_gracefulFs.default, 'readdirAsync').resolves([]);

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      return directory.read().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.readdirAsync).to.have.been.called.with(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'));
      }).finally(function () {
        _gracefulFs.default.readdirAsync.restore();
      });
    });
    it('should should not read the directory while it is already being read', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_gracefulFs.default, 'readdirAsync').resolves([]);

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      directory.read();
      return directory.read().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.readdirAsync).to.have.been.calledOnce();
      }).finally(function () {
        _gracefulFs.default.readdirAsync.restore();
      });
    });
    it('should read any directories contained within the Directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var stat = {
        isDirectory: function isDirectory() {}
      };

      _testSetup.sinon.spy(_Directory.Directory.prototype, 'read');

      var readdirAsync = _testSetup.sinon.stub(_gracefulFs.default, 'readdirAsync');

      readdirAsync.onCall(0).resolves(['directory']);
      readdirAsync.onCall(1).resolves([]);

      _testSetup.sinon.stub(_gracefulFs.default, 'statAsync').resolves(stat);

      _testSetup.sinon.stub(stat, 'isDirectory').returns(true);

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      return directory.read().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.statAsync).to.have.been.called.with(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'));
        (0, _testSetup.expect)(_Directory.Directory.prototype.read).to.have.been.calledTwice();
      }).finally(function () {
        _Directory.Directory.prototype.read.restore();

        _gracefulFs.default.readdirAsync.restore();

        _gracefulFs.default.statAsync.restore();
      });
    });
    it('should create a model for any files contained within the Directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var stat = {
        isDirectory: function isDirectory() {}
      };

      _testSetup.sinon.stub(_gracefulFs.default, 'readdirAsync').resolves(['file.ext']);

      _testSetup.sinon.stub(_gracefulFs.default, 'statAsync').resolves(stat);

      _testSetup.sinon.stub(stat, 'isDirectory').returns(false);

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      return directory.read().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.statAsync).to.have.been.called.with(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory', 'file.ext'));
        (0, _testSetup.expect)(directory.files.length).to.equal(1);
      }).finally(function () {
        _gracefulFs.default.readdirAsync.restore();

        _gracefulFs.default.statAsync.restore();
      });
    });
    it('should create a rich model for files of a known type', function () {
      var TestFile =
      /*#__PURE__*/
      function (_File) {
        _inherits(TestFile, _File);

        function TestFile() {
          _classCallCheck(this, TestFile);

          return _possibleConstructorReturn(this, (TestFile.__proto__ || Object.getPrototypeOf(TestFile)).apply(this, arguments));
        }

        return TestFile;
      }(_File3.File);

      TestFile.prototype.extension = '.ext';
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      fileStructure.addFileType(TestFile);
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      var stat = {
        isDirectory: function isDirectory() {}
      };

      _testSetup.sinon.stub(_File3.File.prototype, 'read');

      _testSetup.sinon.stub(_gracefulFs.default, 'readdirAsync').resolves(['file.ext']);

      _testSetup.sinon.stub(_gracefulFs.default, 'statAsync').resolves(stat);

      _testSetup.sinon.stub(stat, 'isDirectory').returns(false);

      return directory.read().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.statAsync).to.have.been.called.with(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory', 'file.ext'));

        var _directory$files2 = _slicedToArray(directory.files, 1),
            file = _directory$files2[0];

        (0, _testSetup.expect)(file instanceof TestFile).to.equal(true);
      }).finally(function () {
        _File3.File.prototype.read.restore();

        _gracefulFs.default.readdirAsync.restore();

        _gracefulFs.default.statAsync.restore();
      });
    });
    it('should create a rich model for files of a known type with multiple extensions', function () {
      var SpecialTestFile =
      /*#__PURE__*/
      function (_File2) {
        _inherits(SpecialTestFile, _File2);

        function SpecialTestFile() {
          _classCallCheck(this, SpecialTestFile);

          return _possibleConstructorReturn(this, (SpecialTestFile.__proto__ || Object.getPrototypeOf(SpecialTestFile)).apply(this, arguments));
        }

        return SpecialTestFile;
      }(_File3.File);

      SpecialTestFile.prototype.extension = '.special.ext';

      _testSetup.sinon.stub(SpecialTestFile.prototype, 'save').resolves();

      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      fileStructure.addFileType(SpecialTestFile);
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      var stat = {
        isDirectory: function isDirectory() {}
      };

      _testSetup.sinon.stub(_File3.File.prototype, 'read');

      _testSetup.sinon.stub(_gracefulFs.default, 'readdirAsync').resolves(['file.special.ext']);

      _testSetup.sinon.stub(_gracefulFs.default, 'statAsync').resolves(stat);

      _testSetup.sinon.stub(stat, 'isDirectory').returns(false);

      return directory.read().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.statAsync).to.have.been.called.with(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory', 'file.special.ext'));

        var _directory$files3 = _slicedToArray(directory.files, 1),
            file = _directory$files3[0];

        (0, _testSetup.expect)(file instanceof SpecialTestFile).to.equal(true);
      }).finally(function () {
        _File3.File.prototype.read.restore();

        _gracefulFs.default.readdirAsync.restore();

        _gracefulFs.default.statAsync.restore();
      });
    });
  });
  describe('Directory.refresh:', function () {
    it('should refresh the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'init');

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'read').resolves();

      return directory.refresh().then(function () {
        (0, _testSetup.expect)(_Directory.Directory.prototype.init).to.have.been.called();
        (0, _testSetup.expect)(_Directory.Directory.prototype.read).to.have.been.called();
      }).finally(function () {
        _Directory.Directory.prototype.init.restore();

        _Directory.Directory.prototype.read.restore();
      });
    });
    it('should should not refresh while the directory is being read', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'read').resolves();

      directory.refresh();
      return directory.refresh().then(function () {
        (0, _testSetup.expect)(_Directory.Directory.prototype.read).to.have.been.calledOnce();
      }).finally(function () {
        _Directory.Directory.prototype.read.restore();
      });
    });
  });
  describe('Directory.removeItem:', function () {
    it('should remove a file from the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      directory.removeItem(file);
      (0, _testSetup.expect)(directory.files.length).to.equal(0);
    });
    it('should remove the file from the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(fileStructure, 'removeItem');

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      directory.files.push(file);
      directory.removeItem(file);
      (0, _testSetup.expect)(fileStructure.removeItem).to.have.been.calledWith(file);
    });
    it('should remove a directory from the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);
      directory.removeItem(subdirectory);
      (0, _testSetup.expect)(directory.directories.length).to.equal(0);
    });
    it('should remove the directory from the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(fileStructure, 'removeItem');

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);
      directory.removeItem(subdirectory);
      (0, _testSetup.expect)(fileStructure.removeItem).to.have.been.calledWith(subdirectory);
    });
  });
  describe('Directory.rimraf:', function () {
    it('should delete the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      return directory.rimraf().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.rmdirAsync).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'));
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should remove itself from its parent', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var parent = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.stub(parent, 'removeItem');

      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      return directory.rimraf().then(function () {
        (0, _testSetup.expect)(parent.removeItem).to.have.been.calledWith(directory);
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should remove itself from the FileStructure if it is the root direction', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.stub(fileStructure, 'removeItem');

      return fileStructure.structure.rimraf().then(function () {
        (0, _testSetup.expect)(fileStructure.removeItem).to.have.been.calledWith(fileStructure.structure);
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should delete all its sub-directories', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory', 'sub-directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.stub(subdirectory, 'rimraf');

      return directory.rimraf().then(function () {
        (0, _testSetup.expect)(subdirectory.rimraf).to.have.been.called();
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
    it('should delete all its files', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory', 'file.ext'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'rmdirAsync').resolves();

      _testSetup.sinon.stub(file, 'delete');

      return directory.rimraf().then(function () {
        (0, _testSetup.expect)(file.delete).to.have.been.called();
      }).finally(function () {
        _gracefulFs.default.rmdirAsync.restore();
      });
    });
  });
  describe('Directory.save:', function () {
    it('should should do nothing if the directory already exists', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'statAsync').resolves();

      _testSetup.sinon.spy(_gracefulFs.default, 'mkdirAsync');

      return directory.save().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.statAsync).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'));
        (0, _testSetup.expect)(_gracefulFs.default.mkdirAsync).to.not.have.been.called();
      }).finally(function () {
        _gracefulFs.default.statAsync.restore();

        _gracefulFs.default.mkdirAsync.restore();
      });
    });
    it("should save the directory if it doesn't exist yet", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'statAsync').rejects();

      _testSetup.sinon.stub(_gracefulFs.default, 'mkdirAsync').resolves();

      return directory.save().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.mkdirAsync).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory'));
      }).finally(function () {
        _gracefulFs.default.mkdirAsync.restore();
      });
    });
  });
  describe('Directory.toJSON:', function () {
    it('should return important properties of the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subdirectory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      (0, _testSetup.expect)(directory.toJSON()).to.deep.equal({
        basename: 'directory',
        directories: [subdirectory.serialise()],
        files: [file.serialise()],
        isDirectory: true,
        path: _path.default.join(_path.default.sep, 'file-structure', 'directory'),
        url: '/directory'
      });
    });
    it('should order the directories in alphabetic order', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var subdirectory1 = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory-z'), fileStructure);
      var subdirectory2 = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory-a'), fileStructure);
      var subdirectory3 = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory-f'), fileStructure);
      var subdirectory4 = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'sub-directory-a'), fileStructure);
      (0, _testSetup.expect)(directory.toJSON()).to.deep.equal({
        basename: 'directory',
        directories: [subdirectory2.serialise(), subdirectory4.serialise(), subdirectory3.serialise(), subdirectory1.serialise()],
        files: [],
        isDirectory: true,
        path: _path.default.join(_path.default.sep, 'file-structure', 'directory'),
        url: '/directory'
      });
    });
    it('should order the files in alphabetic order', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file1 = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file-z.ext'), fileStructure);
      var file2 = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file-a.ext'), fileStructure);
      var file3 = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file-f.ext'), fileStructure);
      var file4 = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file-a.ext'), fileStructure);
      (0, _testSetup.expect)(directory.toJSON()).to.deep.equal({
        basename: 'directory',
        directories: [],
        files: [file2.serialise(), file4.serialise(), file3.serialise(), file1.serialise()],
        isDirectory: true,
        path: _path.default.join(_path.default.sep, 'file-structure', 'directory'),
        url: '/directory'
      });
    });
  });
});