"use strict";

var _testSetup = require("../../test-setup");

var _bluebird = _interopRequireDefault(require("bluebird"));

var _gracefulFs = _interopRequireDefault(require("graceful-fs"));

var _path = _interopRequireDefault(require("path"));

var _Directory = require("./Directory");

var _FileStructure = require("./FileStructure");

var _ReferenceManager = require("./ReferenceManager");

var _errorHandler = require("@tractor/error-handler");

var _File3 = require("./File");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

describe('@tractor/file-structure - File:', function () {
  describe('File constructor:', function () {
    it('should create a new File', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'file.extension'), fileStructure);
      (0, _testSetup.expect)(file).to.be.an.instanceof(_File3.File);
    });
    it('should work out the name', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      (0, _testSetup.expect)(file.name).to.equal('file.ext');
    });
    it('should get the extension from the file', function () {
      var TestFile =
      /*#__PURE__*/
      function (_File) {
        _inherits(TestFile, _File);

        function TestFile() {
          _classCallCheck(this, TestFile);

          return _possibleConstructorReturn(this, (TestFile.__proto__ || Object.getPrototypeOf(TestFile)).apply(this, arguments));
        }

        return TestFile;
      }(_File3.File);

      TestFile.prototype.extension = '.test.ext';
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new TestFile(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.test.ext'), fileStructure);
      (0, _testSetup.expect)(file.extension).to.equal('.test.ext');
    });
    it('should fall back to getting the extension from the path', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      (0, _testSetup.expect)(file.extension).to.equal('.ext');
    });
    it('should work out the basename from the path and the extension', function () {
      var TestFile =
      /*#__PURE__*/
      function (_File2) {
        _inherits(TestFile, _File2);

        function TestFile() {
          _classCallCheck(this, TestFile);

          return _possibleConstructorReturn(this, (TestFile.__proto__ || Object.getPrototypeOf(TestFile)).apply(this, arguments));
        }

        return TestFile;
      }(_File3.File);

      TestFile.prototype.extension = '.test.ext';
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new TestFile(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.test.ext'), fileStructure);
      (0, _testSetup.expect)(file.basename).to.equal('file');
    });
    it('should work out the URL to the file from the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      (0, _testSetup.expect)(file.url).to.equal('/directory/file.ext');
    });
    it('should correctly replace Windows path seperators in the URL', function () {
      var origPath = {};
      Object.keys(_path.default.win32).forEach(function (key) {
        origPath[key] = _path.default[key];
        _path.default[key] = _path.default.win32[key];
      });

      _testSetup.sinon.stub(process, 'cwd').returns('');

      var fileStructure = new _FileStructure.FileStructure(_path.default.win32.join(_path.default.win32.sep, 'file-structure'));
      var file = new _File3.File(_path.default.win32.join(_path.default.win32.sep, 'file-structure', 'directory', 'another-directory', 'file.ext'), fileStructure);
      (0, _testSetup.expect)(file.url).to.equal('/directory/another-directory/file.ext');
      process.cwd.restore();
      Object.keys(_path.default.win32).forEach(function (key) {
        _path.default[key] = origPath[key];
      });
    });
    it('should work out the parent directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      (0, _testSetup.expect)(file.directory).to.equal(directory);
    });
    it("should create the parent directory if it doesn't exist", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory', 'file.ext'), fileStructure);
      (0, _testSetup.expect)(file.directory).to.not.be.undefined();
      (0, _testSetup.expect)(fileStructure.allDirectoriesByPath[_path.default.join(_path.default.sep, 'file-structure', 'parent-directory')]).to.not.be.undefined();
      (0, _testSetup.expect)(fileStructure.allDirectoriesByPath[_path.default.join(_path.default.sep, 'file-structure', 'parent-directory', 'directory')]).to.not.be.undefined();
    });
    it('should throw an error if the File path is outside the root of the FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      (0, _testSetup.expect)(function () {
        new _File3.File(_path.default.join(_path.default.sep, 'outside', 'file.ext'), fileStructure);
      }).to.throw(_errorHandler.TractorError, "Cannot create \"".concat(_path.default.join(_path.default.sep, 'outside', 'file.ext'), "\" because it is outside of the root of the FileStructure"));
    });
    it('should throw an error if the File path is outside the root of the FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      (0, _testSetup.expect)(function () {
        new _File3.File(_path.default.join(_path.default.sep, 'outside', 'file.ext'), fileStructure);
      }).to.throw(_errorHandler.TractorError, "Cannot create \"".concat(_path.default.join(_path.default.sep, 'outside', 'file.ext'), "\" because it is outside of the root of the FileStructure"));
    });
    it('should be added to its directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(directory, 'addItem');

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      (0, _testSetup.expect)(directory.addItem).to.have.been.calledWith(file);
    });
  });
  describe('File.addReference', function () {
    it('should add a reference from one file to the other', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'file.ext'), fileStructure);
      var otherFile = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'other file.ext'), fileStructure);
      file.addReference(otherFile);
      (0, _testSetup.expect)(file.references).to.deep.equal([otherFile]);
      (0, _testSetup.expect)(otherFile.referencedBy).to.deep.equal([file]);
    });
  });
  describe('File.cleanup:', function () {
    it('should delete the directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(directory, 'delete').resolves();

      _testSetup.sinon.stub(directory.directory, 'cleanup').resolves();

      return directory.cleanup().then(function () {
        (0, _testSetup.expect)(directory.delete).to.have.been.called();
      });
    });
    it('should cleanup the parent directory', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'file.ext'), fileStructure);

      _testSetup.sinon.stub(file, 'delete').resolves();

      _testSetup.sinon.stub(file.directory, 'cleanup').resolves();

      return file.cleanup().then(function () {
        (0, _testSetup.expect)(file.directory.cleanup).to.have.been.called();
      });
    });
    it('should stop once it gets to a directory that is not empty', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      var file1 = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file-1.ext'), fileStructure);
      var file2 = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file-2.ext'), fileStructure);

      _testSetup.sinon.stub(_gracefulFs.default, 'unlinkAsync').resolves();

      _testSetup.sinon.spy(directory, 'delete');

      _testSetup.sinon.spy(file1, 'delete');

      _testSetup.sinon.spy(file2, 'delete');

      return file1.cleanup().then(function () {
        (0, _testSetup.expect)(file1.delete).to.have.been.called();
        (0, _testSetup.expect)(_gracefulFs.default.unlinkAsync).to.have.been.calledOnce();
        (0, _testSetup.expect)(directory.delete).to.have.been.called();
        (0, _testSetup.expect)(file2.delete).to.not.have.been.called();
      }).finally(function () {
        _gracefulFs.default.unlinkAsync.restore();
      });
    });
    it('should rethrow if something unexpected goes wrong', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'file.ext'), fileStructure);
      var error = new Error('Unexpected error');

      _testSetup.sinon.stub(_gracefulFs.default, 'unlinkAsync').rejects(error);

      return file.cleanup().catch(function (e) {
        (0, _testSetup.expect)(e).to.equal(error);
      }).finally(function () {
        _gracefulFs.default.unlinkAsync.restore();
      });
    });
  });
  describe('File.clearReferences', function () {
    it('should clear all references to and from a file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'file.ext'), fileStructure);
      var otherFile = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file.ext'), fileStructure);
      file.addReference(otherFile);
      (0, _testSetup.expect)(file.references).to.deep.equal([otherFile]);
      (0, _testSetup.expect)(otherFile.referencedBy).to.deep.equal([file]);
      file.clearReferences();
      (0, _testSetup.expect)(file.references).to.deep.equal([]);
      (0, _testSetup.expect)(otherFile.referencedBy).to.deep.equal([]);
    });
  });
  describe('File.delete:', function () {
    it('should delete the file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_gracefulFs.default, 'unlinkAsync').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.delete().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.unlinkAsync).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'));
      }).finally(function () {
        _gracefulFs.default.unlinkAsync.restore();
      });
    });
    it("should remove the file from it's directory", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(directory, 'removeItem');

      _testSetup.sinon.stub(_gracefulFs.default, 'unlinkAsync').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.delete().then(function () {
        (0, _testSetup.expect)(directory.removeItem).to.have.been.calledOnce();
      }).finally(function () {
        _gracefulFs.default.unlinkAsync.restore();
      });
    });
    it("shouldn't remove the file if it's referenced by another file", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(directory, 'removeItem');

      _testSetup.sinon.stub(_gracefulFs.default, 'unlinkAsync').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var otherFile = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'), fileStructure);
      otherFile.addReference(file);
      return file.delete().catch(function (e) {
        (0, _testSetup.expect)(e).to.deep.equal(new _errorHandler.TractorError("Cannot delete ".concat(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), " as it is referenced by another file.")));
      }).then(function () {
        (0, _testSetup.expect)(directory.removeItem).to.not.have.been.called();
      }).finally(function () {
        _gracefulFs.default.unlinkAsync.restore();
      });
    });
    it("should remove the file if it's being moved", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);

      _testSetup.sinon.stub(directory, 'removeItem');

      _testSetup.sinon.stub(_gracefulFs.default, 'unlinkAsync').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.delete({
        isMove: true
      }).then(function () {
        (0, _testSetup.expect)(directory.removeItem).to.have.been.called();
      }).finally(function () {
        _gracefulFs.default.unlinkAsync.restore();
      });
    });
  });
  describe('File.move:', function () {
    it('should move a file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'file.ext')
      }).then(function () {
        (0, _testSetup.expect)(_File3.File.prototype.constructor).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'file.ext'), fileStructure);
        (0, _testSetup.expect)(_File3.File.prototype.save).to.have.been.called();
        (0, _testSetup.expect)(_File3.File.prototype.delete).to.have.been.called();
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();
      });
    });
    it('should pass the options through to the `save` and `delete` calls', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var buffer = new Buffer('data');
      file.buffer = buffer;
      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'file.ext')
      }).then(function () {
        (0, _testSetup.expect)(_File3.File.prototype.constructor).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'file.ext'), fileStructure);
        (0, _testSetup.expect)(_File3.File.prototype.save).to.have.been.calledWith(buffer, {
          isMove: true
        });
        (0, _testSetup.expect)(_File3.File.prototype.delete).to.have.been.calledWith({
          isMove: true
        });
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();
      });
    });
    it('should copy a file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'file.ext')
      }, {
        isCopy: true
      }).then(function () {
        (0, _testSetup.expect)(_File3.File.prototype.constructor).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'file.ext'), fileStructure);
        (0, _testSetup.expect)(_File3.File.prototype.save).to.have.been.called();
        (0, _testSetup.expect)(_File3.File.prototype.delete).to.not.have.been.called();
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();
      });
    });
    it('should clear all the references', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'clearReferences');

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
      }).then(function () {
        (0, _testSetup.expect)(file.clearReferences).to.have.been.called();
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.clearReferences.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();
      });
    });
    it('should add the reference back', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var reference = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'), fileStructure);

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_ReferenceManager.ReferenceManager.prototype, 'getReferencedBy').returns([reference]);

      _testSetup.sinon.stub(reference, 'addReference');

      _testSetup.sinon.stub(reference, 'refactor').resolves();

      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
      }).then(function () {
        (0, _testSetup.expect)(reference.addReference).to.have.been.called();
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();

        _ReferenceManager.ReferenceManager.prototype.getReferencedBy.restore();
      });
    });
    it("shouldn't clear all the references if it is a copy", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'clearReferences');

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
      }, {
        isCopy: true
      }).then(function () {
        (0, _testSetup.expect)(file.clearReferences).to.not.have.been.called();
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.clearReferences.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();
      });
    });
    it('should call `refactor` on any files that it is reference by', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var reference = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'), fileStructure);

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_ReferenceManager.ReferenceManager.prototype, 'getReferencedBy').returns([reference]);

      _testSetup.sinon.stub(reference, 'refactor').resolves();

      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
      }).then(function () {
        (0, _testSetup.expect)(reference.refactor).to.have.been.calledWith('referenceNameChange', {
          oldName: 'file',
          newName: 'renamed',
          extension: '.ext'
        });
        (0, _testSetup.expect)(reference.refactor).to.have.been.calledWith('referencePathChange', {
          fromPath: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'),
          oldToPath: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'),
          newToPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
        });
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();

        _ReferenceManager.ReferenceManager.prototype.getReferencedBy.restore();
      });
    });
    it('should call `refactor` on the new file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'refactor').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
      }).then(function () {
        (0, _testSetup.expect)(_File3.File.prototype.refactor).to.have.been.calledWith('fileNameChange', {
          oldName: 'file',
          newName: 'renamed'
        });
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.refactor.restore();

        _File3.File.prototype.save.restore();
      });
    });
    it('should call `refactor` on any files that it is reference by', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var reference = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ref'), fileStructure);

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_ReferenceManager.ReferenceManager.prototype, 'getReferencedBy').returns([reference]);

      _testSetup.sinon.stub(reference, 'refactor').resolves();

      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
      }).then(function () {
        (0, _testSetup.expect)(reference.refactor).to.have.been.calledWith('referenceNameChange', {
          oldName: 'file',
          newName: 'renamed',
          extension: '.ref'
        });
        (0, _testSetup.expect)(reference.refactor).to.have.been.calledWith('referencePathChange', {
          fromPath: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ref'),
          oldToPath: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'),
          newToPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
        });
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();

        _ReferenceManager.ReferenceManager.prototype.getReferencedBy.restore();
      });
    });
    it("should throw an error if it can't refactor the reference", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var reference = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'), fileStructure);

      _testSetup.sinon.spy(_File3.File.prototype, 'constructor');

      _testSetup.sinon.stub(_File3.File.prototype, 'save').resolves();

      _testSetup.sinon.stub(_File3.File.prototype, 'delete').resolves();

      _testSetup.sinon.stub(_ReferenceManager.ReferenceManager.prototype, 'getReferencedBy').returns([reference]);

      _testSetup.sinon.stub(reference, 'refactor').rejects();

      return file.move({
        newPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
      }).then(function () {
        (0, _testSetup.expect)(reference.refactor).to.have.been.calledWith('referenceNameChange', {
          oldName: 'file',
          newName: 'renamed',
          extension: '.ext'
        });
        (0, _testSetup.expect)(reference.refactor).to.have.been.calledWith('referencePathChange', {
          fromPath: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'),
          oldToPath: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'),
          newToPath: _path.default.join(_path.default.sep, 'file-structure', 'other-directory', 'renamed.ext')
        });
      }).catch(function (e) {
        (0, _testSetup.expect)(e).to.deep.equal(new _errorHandler.TractorError("Could not update references after moving ".concat(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), ".")));
      }).finally(function () {
        _File3.File.prototype.constructor.restore();

        _File3.File.prototype.delete.restore();

        _File3.File.prototype.save.restore();

        _ReferenceManager.ReferenceManager.prototype.getReferencedBy.restore();
      });
    });
  });
  describe('File.read:', function () {
    it('should read the file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_gracefulFs.default, 'readFileAsync').resolves(new Buffer('content'));

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.read().then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.readFileAsync).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'));
        (0, _testSetup.expect)(file.content).to.equal('content');
        (0, _testSetup.expect)(file.buffer.equals(new Buffer('content'))).to.equal(true);
      }).finally(function () {
        _gracefulFs.default.readFileAsync.restore();
      });
    });
  });
  describe('File.refactor:', function () {
    it('should do nothing by default', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.spy(_bluebird.default, 'resolve');

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.refactor().then(function () {
        (0, _testSetup.expect)(_bluebird.default.resolve).to.have.been.called();
      }).finally(function () {
        _bluebird.default.resolve.restore();
      });
    });
  });
  describe('File.save:', function () {
    it('should save the file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_gracefulFs.default, 'mkdirAsync').resolves();

      _testSetup.sinon.stub(_gracefulFs.default, 'writeFileAsync').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.save('content').then(function () {
        (0, _testSetup.expect)(_gracefulFs.default.writeFileAsync).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), 'content');
        (0, _testSetup.expect)(file.content).to.equal('content');
        (0, _testSetup.expect)(file.buffer.equals(new Buffer('content'))).to.equal(true);
      }).finally(function () {
        _gracefulFs.default.mkdirAsync.restore();

        _gracefulFs.default.writeFileAsync.restore();
      });
    });
    it('should make sure the parent directory is saved', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(_Directory.Directory.prototype, 'save').resolves();

      _testSetup.sinon.stub(_gracefulFs.default, 'writeFileAsync').resolves();

      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      return file.save('content').then(function () {
        (0, _testSetup.expect)(_Directory.Directory.prototype.save).to.have.been.called();
      }).finally(function () {
        _Directory.Directory.prototype.save.restore();

        _gracefulFs.default.writeFileAsync.restore();
      });
    });
  });
  describe('File.serialise:', function () {
    it('should call `File.prototype.toJSON`', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var serialised = file.serialise();
      (0, _testSetup.expect)(serialised).to.deep.equal({
        basename: 'file',
        extension: '.ext',
        path: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'),
        references: [],
        referencedBy: [],
        url: '/directory/file.ext'
      });
    });
  });
  describe('File.toJSON:', function () {
    it('should return specific properties of the object', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var json = file.toJSON();
      (0, _testSetup.expect)(json).to.deep.equal({
        basename: 'file',
        extension: '.ext',
        path: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'),
        references: [],
        referencedBy: [],
        url: '/directory/file.ext'
      });
    });
    it('should contain the files that the file references', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var otherFile = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'), fileStructure);
      file.addReference(otherFile);
      var json = file.toJSON();
      (0, _testSetup.expect)(json.references).to.deep.equal([{
        basename: 'other-file',
        extension: '.ext',
        path: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'),
        url: '/directory/other-file.ext'
      }]);
    });
    it('should contain the files that reference the file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file.ext'), fileStructure);
      var otherFile = new _File3.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'), fileStructure);
      otherFile.addReference(file);
      var json = file.toJSON();
      (0, _testSetup.expect)(json.referencedBy).to.deep.equal([{
        basename: 'other-file',
        extension: '.ext',
        path: _path.default.join(_path.default.sep, 'file-structure', 'directory', 'other-file.ext'),
        url: '/directory/other-file.ext'
      }]);
    });
  });
});