"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FileStructure = void 0;

var _chokidar = _interopRequireDefault(require("chokidar"));

var _events = require("events");

var _path = _interopRequireDefault(require("path"));

var _logger = require("@tractor/logger");

var _Directory = require("./Directory");

var _File = require("./File");

var _ReferenceManager = require("./ReferenceManager");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

// Constants:
var DOT_FILE_REGEX = /(^|[/\\])\../;
var EXTENSION_MATCH_REGEX = /[^.]*(\..*)?/; // Dependencies:

var FileStructure =
/*#__PURE__*/
function () {
  function FileStructure(fsPath) {
    _classCallCheck(this, FileStructure);

    this.fileTypes = {};
    this.path = _path.default.resolve(process.cwd(), fsPath);
    this.init();
  }

  _createClass(FileStructure, [{
    key: "addFileType",
    value: function addFileType(fileConstructor) {
      this.fileTypes[fileConstructor.prototype.extension] = fileConstructor;
    }
  }, {
    key: "addItem",
    value: function addItem(item) {
      var collection = item instanceof _Directory.Directory ? this.allDirectoriesByPath : this.allFilesByPath;
      collection[item.path] = item;
    }
  }, {
    key: "getFileConstructor",
    value: function getFileConstructor(filePath) {
      var fileName = _path.default.basename(filePath);

      var _fileName$match = fileName.match(EXTENSION_MATCH_REGEX),
          _fileName$match2 = _slicedToArray(_fileName$match, 2),
          fullExtension = _fileName$match2[1];

      var extension = _path.default.extname(fileName);

      return this.fileTypes[fullExtension] || this.fileTypes[extension] || _File.File;
    }
  }, {
    key: "init",
    value: function init() {
      this.allFilesByPath = {};
      this.allDirectoriesByPath = {};
      this.structure = new _Directory.Directory(this.path, this);
      this.referenceManager = new _ReferenceManager.ReferenceManager(this);
    }
  }, {
    key: "read",
    value: function read() {
      return this.structure.read();
    }
  }, {
    key: "removeItem",
    value: function removeItem(item) {
      var collection = item instanceof _Directory.Directory ? this.allDirectoriesByPath : this.allFilesByPath;
      collection[item.path] = null;
    }
  }, {
    key: "watch",
    value: function watch() {
      var _this = this;

      (0, _logger.info)("Watching ".concat(this.path, " for changes..."));
      var watcher = new _events.EventEmitter();

      _chokidar.default.watch(this.path, {
        ignored: DOT_FILE_REGEX,
        ignoreInitial: true,
        awaitWriteFinish: true
      }).on('all', function (event, itemPath) {
        var changeDirectory;

        if (itemPath === _this.path) {
          changeDirectory = _this.structure;
        } else {
          changeDirectory = _this.allDirectoriesByPath[_path.default.dirname(itemPath)];
        }

        changeDirectory.refresh().then(function () {
          watcher.emit('change');
        });
      });

      return watcher;
    }
  }]);

  return FileStructure;
}();

exports.FileStructure = FileStructure;