"use strict";

var _testSetup = require("../../test-setup");

var _chokidar = _interopRequireDefault(require("chokidar"));

var _events = require("events");

var _path = _interopRequireDefault(require("path"));

var _Directory = require("./Directory");

var _File2 = require("./File");

var _ReferenceManager = require("./ReferenceManager");

var tractorLogger = _interopRequireWildcard(require("@tractor/logger"));

var _FileStructure = require("./FileStructure");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

describe('@tractor/file-structure - FileStructure:', function () {
  describe('FileStructure constructor:', function () {
    it('should create a new FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      (0, _testSetup.expect)(fileStructure).to.be.an.instanceof(_FileStructure.FileStructure);
    });
    it('should initalise its interal data structures', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      (0, _testSetup.expect)(fileStructure.allFilesByPath).to.deep.equal({});
      (0, _testSetup.expect)(Object.keys(fileStructure.allDirectoriesByPath).length).to.equal(1);
    });
    it('should create the root directory', function () {
      _testSetup.sinon.stub(process, 'cwd').returns('');

      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      (0, _testSetup.expect)(fileStructure.structure).to.be.an.instanceof(_Directory.Directory);
      (0, _testSetup.expect)(fileStructure.structure.path).to.equal(_path.default.join(_path.default.sep, 'file-structure'));
      process.cwd.restore();
    });
    it('should create the references manager', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      (0, _testSetup.expect)(fileStructure.referenceManager).to.be.an.instanceof(_ReferenceManager.ReferenceManager);
    });
  });
  describe('FileStructure.addFileType', function () {
    it('should add a type of file that the FileStructure can create', function () {
      var TestFile =
      /*#__PURE__*/
      function (_File) {
        _inherits(TestFile, _File);

        function TestFile() {
          _classCallCheck(this, TestFile);

          return _possibleConstructorReturn(this, (TestFile.__proto__ || Object.getPrototypeOf(TestFile)).apply(this, arguments));
        }

        _createClass(TestFile, [{
          key: "save",
          value: function save() {}
        }]);

        return TestFile;
      }(_File2.File);

      TestFile.prototype.extension = '.ext';
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      fileStructure.addFileType(TestFile);
      (0, _testSetup.expect)(fileStructure.fileTypes['.ext']).to.equal(TestFile);
    });
  });
  describe('FileStructure.addItem:', function () {
    it('should add a file to the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File2.File(_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file'), fileStructure);
      fileStructure.removeItem(file);
      (0, _testSetup.expect)(fileStructure.allFilesByPath[_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file')]).to.equal(null);
      fileStructure.addItem(file);
      (0, _testSetup.expect)(fileStructure.allFilesByPath[_path.default.join(_path.default.sep, 'file-structure', 'directory', 'file')]).to.equal(file);
    });
    it('should add a directory to the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      fileStructure.removeItem(directory);
      (0, _testSetup.expect)(fileStructure.allDirectoriesByPath[_path.default.join(_path.default.sep, 'file-structure', 'directory')]).to.equal(null);
      fileStructure.addItem(directory);
      (0, _testSetup.expect)(fileStructure.allDirectoriesByPath[_path.default.join(_path.default.sep, 'file-structure', 'directory')]).to.equal(directory);
    });
  });
  describe('FileStructure.read', function () {
    it('should read the entire file structure', function () {
      _testSetup.sinon.stub(_Directory.Directory.prototype, 'read').resolves();

      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      return fileStructure.read().then(function () {
        (0, _testSetup.expect)(_Directory.Directory.prototype.read).to.have.been.called();
      }).finally(function () {
        _Directory.Directory.prototype.read.restore();
      });
    });
  });
  describe('FileStructure.removeItem:', function () {
    it('should remove a file from the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File2.File(_path.default.join(_path.default.sep, 'file-structure', 'file.extension'), fileStructure);
      (0, _testSetup.expect)(fileStructure.allFilesByPath[_path.default.join(_path.default.sep, 'file-structure', 'file.extension')]).to.equal(file);
      fileStructure.removeItem(file);
      (0, _testSetup.expect)(fileStructure.allFilesByPath[_path.default.join(_path.default.sep, 'file-structure', 'file.extension')]).to.be.null();
    });
    it('should remove a directory from the fileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var directory = new _Directory.Directory(_path.default.join(_path.default.sep, 'file-structure', 'directory'), fileStructure);
      (0, _testSetup.expect)(fileStructure.allDirectoriesByPath[_path.default.join(_path.default.sep, 'file-structure', 'directory')]).to.equal(directory);
      fileStructure.removeItem(directory);
      (0, _testSetup.expect)(fileStructure.allDirectoriesByPath[_path.default.join(_path.default.sep, 'file-structure', 'directory')]).to.be.null();
    });
  });
  describe('FileStructure.watch:', function () {
    it('should set up a watcher on the file structure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));

      _testSetup.sinon.stub(tractorLogger, 'info');

      _testSetup.sinon.stub(_chokidar.default, 'watch').returns(new _events.EventEmitter());

      fileStructure.watch();
      (0, _testSetup.expect)(_chokidar.default.watch).to.have.been.calledWith(_path.default.join(_path.default.sep, 'file-structure'));

      _chokidar.default.watch.restore();

      tractorLogger.info.restore();
    });
    it('should refresh the FileStructure when a file within it changes', function (done) {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File2.File(_path.default.join(_path.default.sep, 'file-structure', 'file.extension'), fileStructure);
      var eventEmitter = new _events.EventEmitter();

      _testSetup.sinon.stub(fileStructure.structure, 'refresh').resolves();

      _testSetup.sinon.stub(tractorLogger, 'info');

      _testSetup.sinon.stub(_chokidar.default, 'watch').returns(eventEmitter);

      var watcher = fileStructure.watch();
      eventEmitter.emit('all', null, file.path);
      watcher.on('change', function () {
        (0, _testSetup.expect)(fileStructure.structure.refresh).to.have.been.called();

        _chokidar.default.watch.restore();

        tractorLogger.info.restore();
        done();
      });
    });
    it('should refresh the FileStructure when the root changes', function (done) {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var eventEmitter = new _events.EventEmitter();

      _testSetup.sinon.stub(fileStructure.structure, 'refresh').resolves();

      _testSetup.sinon.stub(tractorLogger, 'info');

      _testSetup.sinon.stub(_chokidar.default, 'watch').returns(eventEmitter);

      var watcher = fileStructure.watch();
      eventEmitter.emit('all', null, fileStructure.path);
      watcher.on('change', function () {
        (0, _testSetup.expect)(fileStructure.structure.refresh).to.have.been.called();

        _chokidar.default.watch.restore();

        tractorLogger.info.restore();
        done();
      });
    });
  });
});