"use strict";

var _testSetup = require("../../test-setup");

var _path = _interopRequireDefault(require("path"));

var _File = require("./File");

var _FileStructure = require("./FileStructure");

var _ReferenceManager = require("./ReferenceManager");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global describe:true, it:true */
// Test setup:
// Dependencies:
// Under test:
describe('@tractor/file-structure - ReferenceManager:', function () {
  describe('ReferenceManager constructor:', function () {
    it('should create a new ReferenceManager', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var referenceManager = new _ReferenceManager.ReferenceManager(fileStructure);
      (0, _testSetup.expect)(referenceManager).to.be.an.instanceof(_ReferenceManager.ReferenceManager);
    });
  });
  describe('ReferenceManager.addFileStructure', function () {
    it('should add another FileStructure to the ReferenceManager', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var referenceManager = new _ReferenceManager.ReferenceManager(fileStructure);
      var otherFileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'other-file-structure'));
      referenceManager.addFileStructure(otherFileStructure);
      (0, _testSetup.expect)(referenceManager.getFileStructures()).to.deep.equal([fileStructure, otherFileStructure]);
    });
    it('should add the ReferenceManager to the other FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var referenceManager = new _ReferenceManager.ReferenceManager(fileStructure);
      var otherFileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'other-file-structure'));
      referenceManager.addFileStructure(otherFileStructure);
      (0, _testSetup.expect)(otherFileStructure.referenceManager.getFileStructures()).to.deep.equal([otherFileStructure, fileStructure]);
    });
  });
  describe('ReferenceManager.addReference', function () {
    describe('references', function () {
      it('should add a reference to the file from the other file', function () {
        var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
        var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
        var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var referenceManager = fileStructure.referenceManager;
        referenceManager.addReference(file, otherFile);
        (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([otherFile]);
      });
      it('should only be added once', function () {
        var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
        var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
        var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var referenceManager = fileStructure.referenceManager;
        referenceManager.addReference(file, otherFile);
        referenceManager.addReference(file, otherFile);
        (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([otherFile]);
      });
      it('should only be added once when there are two files objects with the same path', function () {
        var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
        var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
        var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var otherFileWithSamePath = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var referenceManager = fileStructure.referenceManager;
        referenceManager.addReference(file, otherFile);
        referenceManager.addReference(file, otherFileWithSamePath);
        (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([otherFile]);
      });
      it('should append to the list of referencesTo', function () {
        var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
        var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
        var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var oneMoreFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'one-more-file'), fileStructure);
        var referenceManager = fileStructure.referenceManager;
        referenceManager.addReference(file, otherFile);
        referenceManager.addReference(file, oneMoreFile);
        (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([otherFile, oneMoreFile]);
      });
    });
    describe('referencedBy', function () {
      it('should add a reference from the file to the other file', function () {
        var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
        var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
        var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var referenceManager = fileStructure.referenceManager;
        referenceManager.addReference(file, otherFile);
        (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([file]);
      });
      it('should only be added once', function () {
        var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
        var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
        var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var referenceManager = fileStructure.referenceManager;
        referenceManager.addReference(file, otherFile);
        referenceManager.addReference(file, otherFile);
        (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([file]);
      });
      it('should only be added once when there are two files objects with the same path', function () {
        var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
        var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
        var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var otherFileWithSamePath = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var referenceManager = fileStructure.referenceManager;
        referenceManager.addReference(file, otherFile);
        referenceManager.addReference(file, otherFileWithSamePath);
        (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([file]);
      });
      it('should append to the list of referencedBy', function () {
        var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
        var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
        var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
        var oneMoreFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'one-more-file'), fileStructure);
        var referenceManager = fileStructure.referenceManager;
        referenceManager.addReference(file, otherFile);
        referenceManager.addReference(oneMoreFile, otherFile);
        (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([file, oneMoreFile]);
      });
    });
  });
  describe('ReferenceManager.getReference', function () {
    it('should get a file from the current FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var referenceManager = new _ReferenceManager.ReferenceManager(fileStructure);
      (0, _testSetup.expect)(referenceManager.getReference(file.path)).to.equal(file);
    });
    it('should get a file from an added FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var otherFileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'other-file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'other-file-structure', 'file'), otherFileStructure);
      var referenceManager = new _ReferenceManager.ReferenceManager(fileStructure);
      referenceManager.addFileStructure(otherFileStructure);
      (0, _testSetup.expect)(referenceManager.getReference(file.path)).to.equal(file);
    });
    it("should return null if the file doesn't exist", function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var referenceManager = new _ReferenceManager.ReferenceManager(fileStructure);
      (0, _testSetup.expect)(referenceManager.getReference(_path.default.join(_path.default.sep, 'some-file'))).to.equal(null);
    });
  });
  describe('ReferenceManager.getReferences', function () {
    it('should return the recorded references to a file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([otherFile]);
    });
    it('should include any references from another FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'other-file-structure'));
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'other-file-structure', 'other-file'), otherFileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addFileStructure(otherFileStructure);
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([otherFile]);
    });
  });
  describe('ReferenceManager.getReferencedBy', function () {
    it('should return the recorded references from a file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([file]);
    });
    it('should include any references from another FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'other-file-structure'));
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'other-file-structure', 'other-file'), otherFileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addFileStructure(otherFileStructure);
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([file]);
    });
  });
  describe('ReferenceManager.clearReferences', function () {
    it('should clear all the references to a file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([otherFile]);
      referenceManager.clearReferences(file.path);
      (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([]);
    });
    it('should clear all the references to a file in another FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'other-file-structure'));
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'other-file-structure', 'other-file'), otherFileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addFileStructure(otherFileStructure);
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([otherFile]);
      referenceManager.clearReferences(file.path);
      (0, _testSetup.expect)(referenceManager.getReferences(file.path)).to.deep.equal([]);
    });
    it('should do nothing if there are no references for a given path', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(function () {
        referenceManager.clearReferences('/some/file/path');
      }).to.not.throw();
    });
  });
  describe('ReferenceManager.clearReferencedBy', function () {
    it('should clear all the references from a file', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([file]);
      referenceManager.clearReferencedBy(otherFile.path);
      (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([]);
    });
    it('should clear all the references from a file in another FileStructure', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'other-file-structure'));
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'other-file-structure', 'other-file'), otherFileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addFileStructure(otherFileStructure);
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([file]);
      referenceManager.clearReferencedBy(otherFile.path);
      (0, _testSetup.expect)(referenceManager.getReferencedBy(otherFile.path)).to.deep.equal([]);
    });
    it('should do nothing if there are no references for a given path', function () {
      var fileStructure = new _FileStructure.FileStructure(_path.default.join(_path.default.sep, 'file-structure'));
      var file = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'file'), fileStructure);
      var otherFile = new _File.File(_path.default.join(_path.default.sep, 'file-structure', 'other-file'), fileStructure);
      var referenceManager = fileStructure.referenceManager;
      referenceManager.addReference(file, otherFile);
      (0, _testSetup.expect)(function () {
        referenceManager.clearReferencedBy('/some/file/path');
      }).to.not.throw();
    });
  });
});