"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.copyFile = copyFile;
exports.createDir = createDir;

var _bluebird = _interopRequireDefault(require("bluebird"));

var _gracefulFs = _interopRequireDefault(require("graceful-fs"));

var _logger = require("@tractor/logger");

var _errorHandler = require("@tractor/error-handler");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Constants:
var ALREADY_EXISTS = 'EEXIST';
var DOESNT_EXIST = 'ENOENT'; // Dependencies:

function copyFile(readPath, writePath) {
  return _gracefulFs.default.readFileAsync(writePath).then(function () {
    return throwExists(writePath);
  }).catch(_bluebird.default.OperationalError, function (error) {
    if (error && error.cause && error.cause.code === DOESNT_EXIST) {
      logCreating(writePath);
    } else {
      throw error;
    }
  }).then(function () {
    return _gracefulFs.default.readFileAsync(readPath);
  }).then(function (contents) {
    return _gracefulFs.default.writeFileAsync(writePath, contents);
  }).then(function () {
    return logCreated(writePath);
  });
}

function createDir(directoryPath) {
  logCreating(directoryPath);
  return _gracefulFs.default.mkdirAsync(directoryPath).then(function () {
    return logCreated(directoryPath);
  }).catch(_bluebird.default.OperationalError, function (error) {
    if (error && error.cause && error.cause.code === ALREADY_EXISTS) {
      throwExists(directoryPath);
    } else {
      throw error;
    }
  });
}

function logCreating(path) {
  (0, _logger.info)("Creating \"".concat(path, "\"..."));
}

function logCreated(path) {
  (0, _logger.info)("\"".concat(path, "\" created."));
}

function throwExists(path) {
  throw new _errorHandler.TractorError("\"".concat(path, "\" already exists."));
}